<?php

namespace Drupal\entity_lru_cache;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\MemoryCache\MemoryCacheInterface;

/**
 * Adapter that directs caching to the LRU cache or the usual memory cache.
 */
class Adapter implements MemoryCacheInterface {

  /**
   * The mode the determines which cache to use.
   */
  protected LruMode $mode;

  public function __construct(protected MemoryCacheInterface $decorated, protected LruMemoryCache $lru, string $mode) {
    $this->mode = LruMode::from($mode);
  }

  /**
   * {@inheritdoc}
   */
  public function get($cid, $allow_invalid = FALSE) {
    return $this->getCache()->get($cid, $allow_invalid);
  }

  /**
   * {@inheritdoc}
   */
  public function getMultiple(&$cids, $allow_invalid = FALSE) {
    return $this->getCache()->getMultiple($cids, $allow_invalid);
  }

  /**
   * {@inheritdoc}
   */
  public function set($cid, $data, $expire = Cache::PERMANENT, array $tags = []): void {
    $this->getCache()->set($cid, $data, $expire, $tags);
  }

  /**
   * {@inheritdoc}
   */
  public function setMultiple(array $items): void {
    $this->getCache()->setMultiple($items);
  }

  /**
   * {@inheritdoc}
   */
  public function delete($cid): void {
    $this->getCache()->delete($cid);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteMultiple(array $cids): void {
    $this->getCache()->deleteMultiple($cids);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteAll(): void {
    $this->getCache()->deleteAll();
  }

  /**
   * {@inheritdoc}
   */
  public function invalidate($cid): void {
    $this->getCache()->invalidate($cid);
  }

  /**
   * {@inheritdoc}
   */
  public function invalidateMultiple(array $cids): void {
    $this->getCache()->invalidateMultiple($cids);
  }

  /**
   * {@inheritdoc}
   */
  public function invalidateAll(): void {
    $this->getCache()->invalidateAll();
  }

  /**
   * {@inheritdoc}
   */
  public function garbageCollection(): void {
    $this->getCache()->garbageCollection();
  }

  /**
   * {@inheritdoc}
   */
  public function removeBin(): void {
    $this->getCache()->removeBin();
  }

  /**
   * {@inheritdoc}
   */
  public function invalidateTags(array $tags): void {
    $this->getCache()->invalidateTags($tags);
  }

  /**
   * Gets the correct memory cache to use depending the mode.
   *
   * @return \Drupal\Core\Cache\MemoryCache\MemoryCacheInterface
   *   The memory cache to use.
   */
  private function getCache(): MemoryCacheInterface {
    return match (TRUE) {
      $this->mode === LruMode::ON, PHP_SAPI === 'cli' && $this->mode === LruMode::CLI_ONLY  => $this->lru,
      default => $this->decorated,
    };
  }

  /**
   * Reset statically cached variables.
   *
   * @see \Drupal\Core\Cache\MemoryBackend::reset()
   */
  public function reset(): void {
    $cache = $this->getCache();
    if (method_exists($cache, 'reset')) {
      $cache->reset();
    }
  }

}
