<?php

declare(strict_types=1);

namespace Drupal\Tests\entity_lru_cache\Kernel;

use Drupal\Core\Cache\MemoryCache\MemoryCache;
use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Routing\RouteObjectInterface;
use Drupal\entity_lru_cache\Adapter;
use Drupal\entity_lru_cache\LruMode;
use Drupal\KernelTests\KernelTestBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Route;

/**
 * @group entity_lru_cache
 */
class ServiceTest extends KernelTestBase {

  /**
   * Sets the lru_mode container parameter.
   */
  protected LruMode $lruMode = LruMode::ON;

  /**
   * {@inheritdoc}
   */
  public function register(ContainerBuilder $container): void {
    parent::register($container);
    if ($container->hasDefinition('Drupal\entity_lru_cache\LruMemoryCache')) {
      // Alias so we can get at the private service.
      $container->setAlias('test.entity_lru.cache', 'Drupal\entity_lru_cache\LruMemoryCache');
      // Set the LRU mode to the desired value for the test.
      $container->setParameter('lru_mode', $this->lruMode->value);
    }
  }

  /**
   * Tests the revision history controller.
   */
  public function testService(): void {
    $this->assertInstanceOf(MemoryCache::class, $this->container->get('entity.memory_cache'));
    $this->enableModules(['entity_lru_cache']);

    // Test that the LRU Cache is enabled.
    $this->assertInstanceOf(Adapter::class, $this->container->get('entity.memory_cache'));
    $this->assertFalse($this->container->get('test.entity_lru.cache')->get('foo'));
    $this->container->get('entity.memory_cache')->set('foo', 'bar');
    $this->assertSame('bar', $this->container->get('entity.memory_cache')->get('foo')->data);
    $this->assertSame('bar', $this->container->get('test.entity_lru.cache')->get('foo')->data);

    // Rebuild the container and disable the LRU cache.
    $this->lruMode = LruMode::OFF;
    $this->container->get('kernel')->shutdown();
    $this->container->get('kernel')->boot();
    $request = Request::create('/');
    $request->attributes->set(RouteObjectInterface::ROUTE_OBJECT, new Route('<none>'));
    $request->attributes->set(RouteObjectInterface::ROUTE_NAME, '<none>');
    $this->container->get('kernel')->preHandle($request);

    // Test that the LRU Cache is disabled.
    $this->assertInstanceOf(Adapter::class, $this->container->get('entity.memory_cache'));
    $this->assertFalse($this->container->get('test.entity_lru.cache')->get('foo'));
    $this->container->get('entity.memory_cache')->set('foo', 'bar');
    $this->assertSame('bar', $this->container->get('entity.memory_cache')->get('foo')->data);
    $this->assertFalse($this->container->get('test.entity_lru.cache')->get('foo'));

    // Test the special reset method.
    $this->container->get('entity.memory_cache')->reset();
    $this->assertFalse($this->container->get('entity.memory_cache')->get('foo'));
  }

}
